package com.bocloud.mcs.collector.alicloud;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.cms20190101.Client;
import com.aliyun.cms20190101.models.DescribeMetricLastRequest;
import com.aliyun.cms20190101.models.DescribeMetricLastResponse;
import com.aliyun.teaopenapi.models.Config;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.alicloud.model.Authentication;
import com.bocloud.mcs.collector.alicloud.model.Datapoints;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * 阿里云监控
 *
 * @author fanzhenxi
 */
@Component
public class AliCloudCollector extends BocloudCollector implements InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(AliCloudCollector.class);
    private final static String PREFIX = "aliyun_";
    Map<String, Object> cache = new HashMap<>();

    /**
     * ECS监控固定写法 acs_ecs_dashboard
     */
    private final String NAMESPACE = "acs_ecs_dashboard";

    @Autowired
    private CurrentService currentService;

    public final static Map<String, String> METRIC_NAME_MAP = new HashMap();
    public final static Map<String, String> AGENT_METRIC_NAME_MAP = new HashMap();
    public final static Map<String, String> REGION_ENDPOINT_MAP = new HashMap();

    static {
        // 基础监控
        METRIC_NAME_MAP.put("CPUUtilization", "CPU使用率");
        METRIC_NAME_MAP.put("DiskReadIOPS", "系统读IOPS");
        METRIC_NAME_MAP.put("DiskWriteIOPS", "系统写IOPS");
        METRIC_NAME_MAP.put("DiskReadBPS", "系统盘总读BPS");
        METRIC_NAME_MAP.put("DiskWriteBPS", "系统盘总写BPS");
        METRIC_NAME_MAP.put("InternetInRate", "公网流入带宽");
        METRIC_NAME_MAP.put("InternetOutRate", "公网流出带宽");
        METRIC_NAME_MAP.put("IntranetInRate", "内网流入带宽");
        METRIC_NAME_MAP.put("IntranetOutRate", "内网流出带宽");
        METRIC_NAME_MAP.put("concurrentConnections", "ECS同时连接数");
        METRIC_NAME_MAP.put("packetInDropRates", "入方向前后端丢包率");
        METRIC_NAME_MAP.put("packetOutDropRates", "出方向前后端丢包率");
        // Agent监控
        AGENT_METRIC_NAME_MAP.put("diskusage_used", "已使用磁盘使用率");
        AGENT_METRIC_NAME_MAP.put("disk_readiops", "磁盘读取IOPS");
        AGENT_METRIC_NAME_MAP.put("disk_writeiops", "磁盘写入IOPS");
        AGENT_METRIC_NAME_MAP.put("diskusage_total", "磁盘总使用率");
        AGENT_METRIC_NAME_MAP.put("disk_readbytes", "磁盘读取字节率");
        AGENT_METRIC_NAME_MAP.put("disk_writebytes", "磁盘写入字节率");
        AGENT_METRIC_NAME_MAP.put("diskusage_avail", "可用磁盘使用率");
        AGENT_METRIC_NAME_MAP.put("diskusage_utilization", "磁盘利用率");
        AGENT_METRIC_NAME_MAP.put("memory_usedutilization", "内存利用率");
        AGENT_METRIC_NAME_MAP.put("networkin_rate", "网络接入率");
        AGENT_METRIC_NAME_MAP.put("networkout_rate", "网络输出率");
        AGENT_METRIC_NAME_MAP.put("networkin_packages", "网络输入包");
        AGENT_METRIC_NAME_MAP.put("networkout_packages", "网络输出包");
        AGENT_METRIC_NAME_MAP.put("networkin_errorpackages", "网络输入错误包");
        AGENT_METRIC_NAME_MAP.put("networkout_errorpackages", "网络输出错误包");
        // 地域endpoint列表
        REGION_ENDPOINT_MAP.put("cn-qingdao","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-beijing","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-zhangjiakou","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-huhehaote","metrics.cn-huhehaote.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-wulanchabu","metrics.cn-wulanchabu.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-hangzhou","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-shanghai","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-shenzhen","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-heyuan","metrics.cn-heyuan.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-guangzhou","metrics.cn-guangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-chengdu","metrics.cn-chengdu.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-hongkong","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("ap-northeast-1","metrics.ap-northeast-1.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("ap-southeast-1","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("ap-southeast-2","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("ap-southeast-3","metrics.ap-southeast-3.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("ap-southeast-5","metrics.ap-southeast-5.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("us-east-1","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("us-west-1","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("eu-west-1","metrics.eu-west-1.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("eu-central-1","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("me-east-1","metrics.cn-hangzhou.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("ap-south-1","metrics.ap-south-1.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-shanghai-finance-1","metrics.cn-shanghai-finance-1.aliyuncs.com");
        REGION_ENDPOINT_MAP.put("cn-shenzhen-finance-1","metrics.cn-shenzhen-finance-1.aliyuncs.com");
    }

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.aliyun.endpoint:}")
    String endpoint;

    @Async
    @Override
    @Scheduled(initialDelay = 1000L, fixedRate = 120000L)
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Date now = new Date();
        logger.info("collect aliyun metrics start...");
        try {
            // 获取bocloudResult对象
            BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.ALIYUN);
            if (bocloudResult.isFailed()) {
                logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
                return;
            }
            JSONObject data = (JSONObject) bocloudResult.getData();
            List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
            Map<String, String> metricMap = new HashMap<>(16);
            metricMap.putAll(METRIC_NAME_MAP);
            metricMap.putAll(AGENT_METRIC_NAME_MAP);
            for (CloudVendor cloudVendor : listCloudVendor) {
                if (StringUtils.isBlank(cloudVendor.getAuthentication())) {
                    continue;
                }
                try {
                    String authentication = cloudVendor.getAuthentication();
                    Authentication authen = JSONArray.parseObject(authentication, Authentication.class);
                    // 判断是否是专有云
                    String address = authen.getAddress();
                    List<String> regions = authen.getRegions();
                    String accessKey = authen.getAccessKey();
                    String secretKey = authen.getSecretKey();
                    accessKey = new DESEncrptor().decrypt(accessKey.trim(), "BocloudCMP");
                    secretKey = new DESEncrptor().decrypt(secretKey.trim(), "BocloudCMP");
                    for (String region : regions) {
                        Config config = new Config()
                                .setAccessKeyId(accessKey)
                                .setAccessKeySecret(secretKey)
                                .setRegionId(region);
                        if (address != null) {
                            // 如果不为null 那么说明是专有云 把地址拼进去
                            if (StringUtils.isNotBlank(endpoint)) {
                                config.setEndpoint(endpoint);
                            } else {
                                config.setEndpoint("metrics." + address);
                            }
                        } else {
                            config.setEndpoint(REGION_ENDPOINT_MAP.get(region));
                        }
                        Client aliClient = new Client(config);
                        for (String metricName : metricMap.keySet()) {
                            DescribeMetricLastRequest request = new DescribeMetricLastRequest();
                            DescribeMetricLastResponse response;
                            // 指标采集
                            GaugeMetricFamily gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.replace(".", "_"), "help", Arrays.asList("vendorId", "instance_id"));
                            GaugeMetricFamily gaugeDevice = new GaugeMetricFamily
                                    (PREFIX + metricName.replace(".", "_"), "help", Arrays.asList("vendorId", "instance_id","device"));
                            GaugeMetricFamily gaugeDiskname = new GaugeMetricFamily
                                    (PREFIX + metricName.replace(".", "_"), "help", Arrays.asList("vendorId", "instance_id","device","diskname","hostname"));
                            GaugeMetricFamily gaugeIP = new GaugeMetricFamily
                                    (PREFIX + metricName.replace(".", "_"), "help", Arrays.asList("vendorId", "instance_id","device","hostname","ip"));
                            do {
                                request.setNamespace(NAMESPACE);
                                request.setMetricName(metricName);
                                response = aliClient.describeMetricLast(request);
                                String datapointsBody = response.getBody().getDatapoints();
                                List<Datapoints> datapointsList = JSONArray.parseArray(datapointsBody, Datapoints.class);
                                if (datapointsList == null || datapointsList.isEmpty()) {
                                    continue;
                                }
                                for (Datapoints datapoints : datapointsList) {
                                    // 这三个指标只有最大值
                                    if ("concurrentConnections".equals(metricName) || "packetInDropRates".equals(metricName) || "packetOutDropRates".equals(metricName)) {
                                        gauge.addMetric(Arrays.asList(cloudVendor.getId().toString(), datapoints.getInstanceId()), datapoints.getMaximum());
                                        continue;
                                    }
                                    // device
                                    if ("disk_readiops".equals(metricName) || "disk_writeiops".equals(metricName) || "disk_readbytes".equals(metricName) || "disk_writebytes".equals(metricName)) {
                                        gaugeDevice.addMetric(Arrays.asList(cloudVendor.getId().toString(), datapoints.getInstanceId(),datapoints.getDevice()), datapoints.getAverage());
                                        continue;
                                    }
                                    // device  diskname  hostname
                                    if ("diskusage_total".equals(metricName)|| "diskusage_used".equals(metricName) || "diskusage_avail".equals(metricName) || "diskusage_utilization".equals(metricName)) {
                                        gaugeDiskname.addMetric(Arrays.asList(cloudVendor.getId().toString(), datapoints.getInstanceId(),datapoints.getDevice(),datapoints.getDiskName(),datapoints.getHostname()), datapoints.getAverage());
                                        continue;
                                    }
                                    // device  diskname  IP
                                    if ("networkin_rate".equals(metricName) || "networkout_rate".equals(metricName) || "networkin_packages".equals(metricName)
                                            || "networkout_packages".equals(metricName) || "networkin_errorpackages".equals(metricName) || "networkout_errorpackages".equals(metricName)) {
                                        gaugeIP.addMetric(Arrays.asList(cloudVendor.getId().toString(), datapoints.getInstanceId(),datapoints.getDevice(),datapoints.getHostname(),datapoints.getIP()), datapoints.getAverage());
                                        continue;
                                    }
                                    // 普通的正常添加
                                    gauge.addMetric(Arrays.asList(cloudVendor.getId().toString(), datapoints.getInstanceId()), datapoints.getAverage());
                                }
                                request.setNextToken(response.getBody().getNextToken());
                            } while (response.getBody().getNextToken() != null);
                            mfs.add(gauge);
                            mfs.add(gaugeDevice);
                            mfs.add(gaugeDiskname);
                            mfs.add(gaugeIP);
                        }
                    }
                }catch (Exception e){
                    logger.error("aliyun collect information", e + cloudVendor.getName());
                }
            }
        } catch (Exception e) {
            logger.error("collect aliyun response error ", e);
        }
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect aliyun metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

}