package com.bocloud.mcs.collector.h3c;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.h3c.core.H3CClient;
import com.bocloud.mcs.collector.h3c.model.Butler;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class H3CHostCollector extends BocloudCollector implements InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(H3CHostCollector.class);

    Map<String, Object> cache = new HashMap<>();

    @Value("${collector.h3c.host.enable:true}")
    boolean enable;

    @Autowired
    private CurrentService currentService;

    public String prefix = "h3c_host_";

    @Override
    public int getExpire() {
        return 1800000;
    }

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Scheduled(initialDelay = 1000L, fixedRate = 30000)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Date now = new Date();
        logger.info("collect h3c host metrics start...");

        H3CClient client = null;

        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.H3C);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);

        List<String> hostLabels = Arrays.asList("vendorId", "id", "name", "ip");
        List<String> diskLabels = new ArrayList<>(hostLabels);
        diskLabels.add("device");
        List<String> ifLabels = new ArrayList<>(hostLabels);
        ifLabels.add("ifName");
        ifLabels.add("mac");
        ifLabels.add("type");
        //CPU利用率
        GaugeMetricFamily cpuGauge = new GaugeMetricFamily(prefix + "cpu_rate", "help", hostLabels);
        //内存利用率
        GaugeMetricFamily memGauge = new GaugeMetricFamily(prefix + "mem_rate", "help", hostLabels);

        //磁盘利用率数据
        GaugeMetricFamily diskUsageGauge = new GaugeMetricFamily(prefix + "disk_usage", "help", diskLabels);
        //I/O读吞吐量
        GaugeMetricFamily diskReadStatGauge = new GaugeMetricFamily(prefix + "disk_read_stat", "help", diskLabels);
        //I/O写吞吐量
        GaugeMetricFamily diskWriteStatGauge = new GaugeMetricFamily(prefix + "disk_write_stat", "help", diskLabels);
        //磁盘读IOPS
        GaugeMetricFamily diskReadRequestGauge = new GaugeMetricFamily(prefix + "disk_read_request", "help", diskLabels);
        //磁盘写IOPS
        GaugeMetricFamily diskWriteRequestGauge = new GaugeMetricFamily(prefix + "disk_write_request", "help", diskLabels);
        //磁盘I/O读延迟
        GaugeMetricFamily diskReadLatencyGauge = new GaugeMetricFamily(prefix + "disk_read_latency", "help", diskLabels);
        //磁盘I/O写延迟
        GaugeMetricFamily diskWriteLatencyGauge = new GaugeMetricFamily(prefix + "disk_write_latency", "help", diskLabels);

        //网络接口
        GaugeMetricFamily ifTxBytesRateGauge = new GaugeMetricFamily(prefix + "if_tx_bytes_rate", "help", ifLabels);
        GaugeMetricFamily ifRxBytesRateGauge = new GaugeMetricFamily(prefix + "if_rx_bytes_rate", "help", ifLabels);
        GaugeMetricFamily ifTxPacketsRateGauge = new GaugeMetricFamily(prefix + "if_tx_packets_rate", "help", ifLabels);
        GaugeMetricFamily ifRxPacketsRateGauge = new GaugeMetricFamily(prefix + "if_rx_packets_rate", "help", ifLabels);

        logger.info("获取到h3c平台个数" + listCloudVendor.size());
        for (CloudVendor cloudVendor : listCloudVendor) {
            try {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");

                Assert.notNull(address, "h3c EndPoint is Empty!");
                Butler butler = new Butler();
                butler.setPassword(password);
                butler.setUrl(address);
                butler.setUsername(account);
                client = new H3CClient(butler);

                //查询所有主机
                List<Map<String, String>> hosts = client.hostList();
                for (Map<String, String> host : hosts) {
                    if ("1".equals(host.get("status"))) {
                        Map<String, String> info = client.listHostMonitorInfoById(MapTools.simpleMap("hostId", host.get("id")));
                        List<String> hostLabelValues = Arrays.asList(String.valueOf(cloudVendor.getId()), host.get("id"), host.get("name"), host.get("ip"));

                        //收集disk_device信息,所有磁盘名
                        List<String> deviceNameList = info.keySet().stream().filter(k -> k.startsWith("disk")).map(k -> k.split("->")[1]).distinct().collect(Collectors.toList());
                        deviceNameList.forEach(deviceName -> {
                            List<String> diskLabelValues = new ArrayList<>(hostLabelValues);
                            diskLabelValues.add(deviceName);
                            diskUsageGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->usage")));
                            diskReadStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->rd_stat")));
                            diskWriteStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->wr_stat")));
                            diskReadRequestGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->rd_req")));
                            diskWriteRequestGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->wr_req")));
                            diskReadLatencyGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->rd_latency")));
                            diskWriteLatencyGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->wr_latency")));
                        });
                        //收集网络接口名
                        List<String> ifNameList =info.keySet().stream().filter(k -> k.startsWith("interface")).map(k -> k.split("->")[1]).distinct().collect(Collectors.toList());
                        ifNameList.forEach(ifName -> {
                            List<String> ifLabelValues = new ArrayList<>(hostLabelValues);
                            ifLabelValues.add(ifName);
                            ifLabelValues.add(info.get("interface->" + ifName + "->mac"));
                            ifLabelValues.add(info.get("interface->" + ifName + "->type"));
                            ifRxBytesRateGauge.addMetric(ifLabelValues, Double.parseDouble(info.get("interface->" + ifName + "->rx_bytes_rate")));
                            ifTxBytesRateGauge.addMetric(ifLabelValues, Double.parseDouble(info.get("interface->" + ifName + "->tx_bytes_rate")));
                            ifRxPacketsRateGauge.addMetric(ifLabelValues, Double.parseDouble(info.get("interface->" + ifName + "->rx_packets_rate")));
                            ifTxPacketsRateGauge.addMetric(ifLabelValues, Double.parseDouble(info.get("interface->" + ifName + "->tx_packets_rate")));
                        });
                        cpuGauge.addMetric(hostLabelValues, Double.parseDouble(info.get("host->cpuRate")));
                        memGauge.addMetric(hostLabelValues, Double.parseDouble(info.get("host->memRate")));
                    }
                }
            } catch (Exception e) {
                logger.error("collect h3c host metrics error! ", e);
            }
        }
        mfs.add(cpuGauge);
        mfs.add(memGauge);
        mfs.add(diskUsageGauge);
        mfs.add(diskReadStatGauge);
        mfs.add(diskWriteStatGauge);
        mfs.add(diskReadRequestGauge);
        mfs.add(diskWriteRequestGauge);
        mfs.add(diskReadLatencyGauge);
        mfs.add(diskWriteLatencyGauge);
        mfs.add(ifRxBytesRateGauge);
        mfs.add(ifTxBytesRateGauge);
        mfs.add(ifRxPacketsRateGauge);
        mfs.add(ifTxPacketsRateGauge);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect h3c host metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}