package com.bocloud.mcs.collector.h3c;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.h3c.core.H3CClient;
import com.bocloud.mcs.collector.h3c.model.Butler;
import com.bocloud.mcs.collector.h3c.model.VmDomain;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class H3CVmCollector extends BocloudCollector implements InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(H3CVmCollector.class);
    Map<String, Object> cache = new HashMap<>();
    @Value("${collector.h3c.vm.enable:true}")
    boolean enable;
    public String prefix = "h3c_vm_";

    @Autowired
    private CurrentService currentService;

    @Override
    public int getExpire() {
        return 1800000;
    }

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Scheduled(initialDelay = 1000L, fixedRate = 30000)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Date now = new Date();
        logger.info("collect h3c vm metrics start...");
        H3CClient client = null;

        List<String> vmLabels = Arrays.asList("id", "name", "uuid", "host_id");
        List<String> diskLabels = new ArrayList<>(vmLabels);
        List<String> netLabels = new ArrayList<>(vmLabels);
        List<String> partitionLabels = new ArrayList<>(vmLabels);
        diskLabels.add("device");
        netLabels.add("mac");
        partitionLabels.add("device");
        //CPU利用率
        GaugeMetricFamily cpuGauge = new GaugeMetricFamily(prefix + "cpu_rate", "help", vmLabels);
        //内存利用率
        GaugeMetricFamily memGauge = new GaugeMetricFamily(prefix + "mem_rate", "help", vmLabels);

        //I/O吞吐量。
        GaugeMetricFamily diskIoStatGauge = new GaugeMetricFamily(prefix + "disk_io_stat", "help", diskLabels);
        //I/O读吞吐量
        GaugeMetricFamily diskReadStatGauge = new GaugeMetricFamily(prefix + "disk_read_stat", "help", diskLabels);
        //I/O写吞吐量
        GaugeMetricFamily diskWriteStatGauge = new GaugeMetricFamily(prefix + "disk_write_stat", "help", diskLabels);
        //磁盘读IOPS
        GaugeMetricFamily diskReadRequestGauge = new GaugeMetricFamily(prefix + "disk_read_request", "help", diskLabels);
        //磁盘写IOPS
        GaugeMetricFamily diskWriteRequestGauge = new GaugeMetricFamily(prefix + "disk_write_request", "help", diskLabels);
        //磁盘I/O读延迟
        GaugeMetricFamily diskReadLatencyGauge = new GaugeMetricFamily(prefix + "disk_read_latency", "help", diskLabels);
        //磁盘I/O写延迟
        GaugeMetricFamily diskWriteLatencyGauge = new GaugeMetricFamily(prefix + "disk_write_latency", "help", diskLabels);

        //网卡读流量
        GaugeMetricFamily netReadFlow = new GaugeMetricFamily(prefix + "net_read_flow", "help", netLabels);
        //网卡读报文数
        GaugeMetricFamily netReadPacketsGauge = new GaugeMetricFamily(prefix + "net_read_packets", "help", netLabels);
        //网卡写流量
        GaugeMetricFamily netWriteFlowGauge = new GaugeMetricFamily(prefix + "net_write_flow", "help", netLabels);
        //网卡写报文数
        GaugeMetricFamily netWritePacketsGauge = new GaugeMetricFamily(prefix + "net_write_packets", "help", netLabels);

        //磁盘利用率数据
        GaugeMetricFamily partitionUsageGauge = new GaugeMetricFamily(prefix + "partition_usage", "help", partitionLabels);

        GaugeMetricFamily flowLinkGauge = new GaugeMetricFamily(prefix + "flow_link", "help", vmLabels);


        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.H3C);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        for (CloudVendor cloudVendor : listCloudVendor) {
            try {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");

                Assert.notNull(address, "h3c EndPoint is Empty!");
                Butler butler = new Butler();
                butler.setPassword(password);
                butler.setUrl(address);
                butler.setUsername(account);
                client = new H3CClient(butler);

                // 查询所有虚拟机列表
                List<VmDomain> vmDomains = client.vmList(new HashMap<>());
                for (VmDomain vmDomain : vmDomains) {
                    if ("shutOff".equals(vmDomain.getVmStatus())) {
                        continue;
                    }
                    Map<String, String> info = client.listVmMonitorInfo(MapTools.simpleMap("vmId", vmDomain.getId()));

                    if (!info.isEmpty()) {
                        List<String> vmLabelValues = Arrays.asList(info.get("vm->id"), vmDomain.getName(), vmDomain.getUuid(), vmDomain.getHostId());

                        flowLinkGauge.addMetric(vmLabelValues, Double.parseDouble(info.get("vm->flowLink")));
                        cpuGauge.addMetric(vmLabelValues, Double.parseDouble(info.get("vm->cpuRate")));
                        memGauge.addMetric(vmLabelValues, Double.parseDouble(info.get("vm->memRate")));

                        List<String> netMacList = info.keySet().stream().filter(k -> k.startsWith("net")).map(k -> k.split("->")[1]).distinct().collect(Collectors.toList());
                        netMacList.forEach(macName -> {
                            List<String> netLabelValues = new ArrayList<>(vmLabelValues);
                            netLabelValues.add(macName);
                            netReadFlow.addMetric(netLabelValues, Double.parseDouble(info.get("net->" + macName + "->readFlow")));
                            netReadPacketsGauge.addMetric(netLabelValues, Double.parseDouble(info.get("net->" + macName + "->readPackets")));
                            netWriteFlowGauge.addMetric(netLabelValues, Double.parseDouble(info.get("net->" + macName + "->writeFlow")));
                            netWritePacketsGauge.addMetric(netLabelValues, Double.parseDouble(info.get("net->" + macName + "->readPackets")));
                        });

                        List<String> partitionDeviceList = info.keySet().stream().filter(k -> k.startsWith("partition")).map(k -> k.split("->")[1]).distinct().collect(Collectors.toList());
                        partitionDeviceList.forEach(partitionDeviceName -> {
                            List<String> partitionLabelValues = new ArrayList<>(vmLabelValues);
                            partitionLabelValues.add(partitionDeviceName);
                            partitionUsageGauge.addMetric(partitionLabelValues, Double.parseDouble(info.get("partition->" + partitionDeviceName + "->usage")));
                        });
                        List<String> deviceNameList = info.keySet().stream().filter(k -> k.startsWith("disk")).map(k -> k.split("->")[1]).distinct().collect(Collectors.toList());
                        deviceNameList.forEach(deviceName -> {
                            List<String> diskLabelValues = new ArrayList<>(vmLabelValues);
                            diskLabelValues.add(deviceName);
                            diskIoStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->ioStat")));
                            diskReadStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->readStat")));
                            diskWriteStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->writeStat")));
                            diskReadRequestGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->readReqest")));
                            diskWriteRequestGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->writeReqest")));
                            diskReadLatencyGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->readLatency")));
                            diskWriteLatencyGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk->" + deviceName + "->writeLatency")));
                        });
                    }
                }
            } catch (Exception e) {
                logger.error("collect h3c vm metrics error !", e);
            }
        }
        mfs.add(cpuGauge);
        mfs.add(memGauge);
        mfs.add(diskIoStatGauge);
        mfs.add(diskReadStatGauge);
        mfs.add(diskWriteStatGauge);
        mfs.add(diskReadRequestGauge);
        mfs.add(diskWriteRequestGauge);
        mfs.add(diskReadLatencyGauge);
        mfs.add(diskWriteLatencyGauge);
        mfs.add(netReadFlow);
        mfs.add(netReadPacketsGauge);
        mfs.add(netWriteFlowGauge);
        mfs.add(netWritePacketsGauge);
        mfs.add(partitionUsageGauge);
        mfs.add(flowLinkGauge);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect h3c vm metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    public static void main(String[] args) {
//        List<MetricFamilySamples> mfs = new ArrayList<>();
//        Date now = new Date();
//        H3cClient client = new H3cClient();
//        try {
//            List<VmDomain> vmDomains = client.vmList(MapTools.simpleMap("hostId", "123456"));
//            if (vmDomains.isEmpty()) {
//                return;
//            }
//
//            List<String> vmLabels = Arrays.asList("id", "name", "uuid", "host_id");
//            List<String> diskLabels = new ArrayList<>();
//            List<String> netLabels = new ArrayList<>();
//            List<String> partitionLabels = new ArrayList<>();
//            diskLabels.addAll(vmLabels);
//            netLabels.addAll(vmLabels);
//            partitionLabels.addAll(vmLabels);
//            diskLabels.add("device");
//            netLabels.add("mac");
//            partitionLabels.add("device");
//            //CPU利用率
//            GaugeMetricFamily cpuGauge = new GaugeMetricFamily("h3c" + "vm_cpu_rate", "help", vmLabels);
//            //内存利用率
//            GaugeMetricFamily memGauge = new GaugeMetricFamily("h3c" + "vm_mem_rate", "help", vmLabels);
//
//            //I/O吞吐量。
//            GaugeMetricFamily diskIoStatGauge = new GaugeMetricFamily("h3c" + "disk_io_status", "help", diskLabels);
//            //I/O读吞吐量
//            GaugeMetricFamily diskReadStatGauge = new GaugeMetricFamily("h3c" + "disk_read_status", "help", diskLabels);
//            //I/O写吞吐量
//            GaugeMetricFamily diskWriteStatGauge = new GaugeMetricFamily("h3c" + "disk_write_status", "help", diskLabels);
//            //磁盘读IOPS
//            GaugeMetricFamily diskReadRequestGauge = new GaugeMetricFamily("h3c" + "disk_read_request", "help", diskLabels);
//            //磁盘写IOPS
//            GaugeMetricFamily diskWriteRequestGauge = new GaugeMetricFamily("h3c" + "disk_write_request", "help", diskLabels);
//            //磁盘I/O读延迟
//            GaugeMetricFamily diskReadLatencyGauge = new GaugeMetricFamily("h3c" + "disk_read_latency", "help", diskLabels);
//            //磁盘I/O写延迟
//            GaugeMetricFamily diskWriteLatencyGauge = new GaugeMetricFamily("h3c" + "disk_write_latency", "help", diskLabels);
//
//            //网卡读流量
//            GaugeMetricFamily netReadFlow = new GaugeMetricFamily("h3c" + "net_read_flow", "help", netLabels);
//            //网卡读报文数
//            GaugeMetricFamily netReadPacketsGauge = new GaugeMetricFamily("h3c" + "net_read_packets", "help", netLabels);
//            //网卡写流量
//            GaugeMetricFamily netWriteFlowGauge = new GaugeMetricFamily("h3c" + "net_write_flow", "help", netLabels);
//            //网卡写报文数
//            GaugeMetricFamily netWritePacketsGauge = new GaugeMetricFamily("h3c" + "net_write_packets", "help", netLabels);
//
//            //磁盘利用率数据
//            GaugeMetricFamily partitionUsageGauge = new GaugeMetricFamily("h3c" + "partition_usage", "help", partitionLabels);
//
//
//            for (VmDomain vmDomain : vmDomains) {
//                Map<String, String> info = client.listVmMonitorInfoBy(MapTools.simpleMap("vmId", vmDomain.getId()));
//
//                List<String> vmLabelValues = Arrays.asList(info.get("vm_id"), vmDomain.getName(), vmDomain.getUuid(), vmDomain.getHostId());
//                List<String> diskLabelValues = new ArrayList<>();
//                List<String> netLabelValues = new ArrayList<>();
//                List<String> partitionLabelValues = new ArrayList<>();
//                diskLabelValues.addAll(vmLabelValues);
//                netLabelValues.addAll(vmLabelValues);
//                partitionLabelValues.addAll(vmLabelValues);
//                diskLabelValues.add(info.get("disk_device"));
//                netLabelValues.add(info.get("net_mac"));
//                partitionLabelValues.add(info.get("partition_device"));
//
//
//
//                netReadFlow.addMetric(netLabelValues, Double.parseDouble(info.get("net_readFlow")));
//                netReadPacketsGauge.addMetric(netLabelValues, Double.parseDouble(info.get("net_readPackets")));
//                netWriteFlowGauge.addMetric(netLabelValues, Double.parseDouble(info.get("net_writeFlow")));
//                netWritePacketsGauge.addMetric(netLabelValues, Double.parseDouble(info.get("net_readPackets")));
//                partitionUsageGauge.addMetric(partitionLabelValues, Double.parseDouble(info.get("partition_usage")));
//
//                cpuGauge.addMetric(vmLabelValues, Double.parseDouble(info.get("vm_cpuRate")));
//                memGauge.addMetric(vmLabelValues, Double.parseDouble(info.get("vm_memRate")));
//                diskIoStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_ioStat")));
//                diskReadStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_readStat")));
//                diskWriteStatGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_writeStat")));
//                diskReadRequestGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_readReqest")));
//                diskWriteRequestGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_writeReqest")));
//                diskReadLatencyGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_readLatency")));
//                diskWriteLatencyGauge.addMetric(diskLabelValues, Double.parseDouble(info.get("disk_writeLatency")));
//
//            }
//            mfs.add(cpuGauge);
//            mfs.add(memGauge);
//            mfs.add(diskIoStatGauge);
//            mfs.add(diskReadStatGauge);
//            mfs.add(diskWriteStatGauge);
//            mfs.add(diskReadRequestGauge);
//            mfs.add(diskWriteRequestGauge);
//            mfs.add(diskReadLatencyGauge);
//            mfs.add(diskWriteLatencyGauge);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
    }
}
