package com.bocloud.mcs.collector.h3c.core;

import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.MapTools;

import com.bocloud.mcs.collector.h3c.model.*;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.dom4j.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.*;

@Data
@NoArgsConstructor
public class H3CClient {
    private static final Logger logger = LoggerFactory.getLogger(H3CClient.class);
    private static final String PROTOCOL = "";
    private static final String vmListUrl = "/cas/casrs/vm/vmList";
    private static final String vmIdMonitorUrl = "/cas/casrs/vm/id/{vmId}/monitor";
    private static String hostIdMonitorUrl = "/cas/casrs/host/id/{hostId}/monitor";
    private static String hostListUrl = "/cas/casrs/host/";

    private String endpoint;
    private Integer port;
    private String username;
    private String password;
    private CloseableHttpClient client;
    private boolean accessable;

    public H3CClient(Butler butler) {
        String newUrl = butler.getUrl().toLowerCase();
        String url = "";
        if (newUrl.indexOf("http://") == 0) {
            url = newUrl.substring(7);
        } else if (newUrl.indexOf("https://") == 0) {
            url = newUrl.substring(8);
        }
        String[] endPointAndPort = url.split(":");
        if (endPointAndPort.length == 2) {
            this.endpoint = endPointAndPort[0];
            this.port = Integer.parseInt(endPointAndPort[1]);
            this.username = butler.getUsername();
            this.password = butler.getPassword();
            this.init();
        } else {
            throw new RuntimeException("错误的参数");
        }
    }

    public void init() {
        try {
            CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
            credentialsProvider.setCredentials(new AuthScope(this.endpoint, this.port, "VMC RESTful Web Services"),
                    new UsernamePasswordCredentials(this.username, this.password));
            HttpClientBuilder httpClientBuilder = HttpClientBuilder.create()
                    .setDefaultCredentialsProvider(credentialsProvider);
            client = httpClientBuilder.build();
            accessable = true;
        } catch (Exception e) {
            logger.error("认证失败", e);
            accessable = false;
        }
    }

    /**
     * 查询主机下所有虚拟机列表
     *
     * @param params
     * @return
     */
    public List<VmDomain> vmList(Map<String, Object> params) {
        List<VmDomain> res = new ArrayList<>();
        Document document = null;
        try {
            Result result = doGet(vmListUrl);
            if (null == result) {
                return res;
            }
            String message = result.getMessage();
            document = DocumentHelper.parseText(message);
        } catch (DocumentException e) {
            logger.error("result message convert error" + e);
            return res;
        }
        Element rootElement = document.getRootElement();//list
        List<Element> elements = rootElement.elements();//domain
        for (Element child : elements) {
            res.add(new VmDomain(child.elementText("id"),
                    child.elementText("hostId"),
                    child.elementText("clusterId"),
                    child.elementText("hostPoolId"),
                    child.elementText("name"),
                    child.elementText("title"),
                    child.elementText("hostName"),
                    child.elementText("memory"),
                    child.elementText("cpuRate"),
                    child.elementText("memRate"),
                    child.elementText("cpu"),
                    child.elementText("vmStatus"),
                    child.elementText("uuid"),
                    child.elementText("system"),
                    child.elementText("osDesc"),
                    child.elementText("flag"),
                    child.elementText("type"),
                    child.elementText("deployed"),
                    child.elementText("existRaw"),
                    child.elementText("haStatus"),
                    child.elementText("haManage")
            ));
        }
        return res;
    }

    /**
     * 查询所有主机
     *
     * @return
     * @throws Exception
     */
    public List<Map<String, String>> hostList() {
        List<Map<String, String>> res = new ArrayList<>();
        Document document = null;
        try {
            Result result = doGet(hostListUrl);
            if (null == result) {
                return res;
            }
            String hostMessage = result.getMessage();
            document = DocumentHelper.parseText(hostMessage);
        } catch (DocumentException e) {
            logger.error("result message convert error！" + e);
            return res;
        }
        Element rootElement = document.getRootElement();//list
        List<Element> elements = rootElement.elements();//host
        for (Element ele : elements) {
            Map<String, String> map = new HashMap<>();
            map.put("id", ele.elementText("id"));
            map.put("name", ele.elementText("name"));
            map.put("ip", ele.elementText("ip"));
            map.put("status", ele.elementText("status"));
            res.add(map);
        }
        return res;
    }


    /**
     * 按虚拟机id/uuid查询虚拟机的的监控性能数据
     *
     * @param params
     * @return
     * @throws Exception
     */
    public Map<String, String> listVmMonitorInfo(Map<String, Object> params) {
        HashMap<String, String> map = new HashMap<>();
        Document document = null;
        String url = vmIdMonitorUrl.replace("{vmId}", (CharSequence) params.get("vmId"));
        try {
            Result result = doGet(url);
            if (null == result) {
                return map;
            }
            String monitorMessage = result.getMessage();
            document = DocumentHelper.parseText(monitorMessage);
        } catch (DocumentException e) {
            logger.error("result message convert error" + e);
            return map;
        }
        Element root = document.getRootElement();
        List<Element> elements = root.elements();
        for (Element ele : elements) {
            String name = ele.getName();
            switch (name) {
                case "disk":
                case "partition":
                    String device = ele.elementText("device");
                    for (Element e : ele.elements()) {
                        map.put(name + "->" + device + "->" + e.getName(), e.getText());
                    }
                    break;
                case "net":
                    String mac = ele.elementText("mac");
                    for (Element e : ele.elements()) {
                        map.put(name + "->" + mac + "->" + e.getName(), e.getText());
                    }
                    break;
                default:
                    map.put("vm->" + ele.getName(), ele.getText());
            }
        }
        return map;
    }

    /**
     * 按主机id查询主机性能数据
     *
     * @param params
     * @return
     * @throws Exception
     */
    public Map<String, String> listHostMonitorInfoById(Map<String, Object> params) {
        HashMap<String, String> map = new HashMap<>();
        String url = hostIdMonitorUrl.replace("{hostId}", (CharSequence) params.get("hostId"));
        Document document = null;
        try {
            Result result = doGet(url);
            if (null == result) {
                return map;
            }
            String hostMonitorMessage = result.getMessage();
            document = DocumentHelper.parseText(hostMonitorMessage);
        } catch (DocumentException e) {
            logger.error("result message convert error" + e);
            return map;
        }
        Element root = document.getRootElement();
        List<Element> elements = root.elements();
        for (Element ele : elements) {
            String name = ele.getName();
            if ("disk".equals(name)) {
                String device = ele.elementText("device");
                for (Element e : ele.elements()) {
                    map.put(name + "->" + device + "->" + e.getName(), e.getText());
                }
            }else if("interface".equals(name)){
                String ifName=ele.elementText("name");
                for (Element e : ele.elements()) {
                    map.put(name + "->" + ifName + "->" + e.getName(), e.getText());
                }
            } else {
                map.put("host->" + ele.getName(), ele.getText());
            }
        }
        return map;
    }


    public Result doGet(String url) {
        HttpGet httpGet = new HttpGet("http://" + this.getEndpoint() + ":" + this.getPort() + url);
        httpGet.addHeader("accept", "application/xml");
        CloseableHttpResponse httpResponse = null;
        Result result = null;
        try {
            httpResponse = client.execute(httpGet);
            result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
            result.setMessage(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
        } catch (Exception e) {
            logger.error("请求io异常{}", ExceptionUtils.getStackTrace(e));
        }
        return result;
    }
}
