package com.bocloud.mcs.collector.openstack;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.common.Constants;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.OSClient;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.common.Identifier;
import org.openstack4j.model.compute.Server;
import org.openstack4j.openstack.OSFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * openStack虚拟机监控采集器
 * @author fanzhenxi
 * @version 1.0
 * @date 2021-03-01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class OpenStackVmCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(OpenStackVmCollector.class);

    protected static Config config;

    static {
        // 全部信任 不做身份鉴定
        try {
            SSLContextBuilder contextBuilder = SSLContextBuilder.create().loadTrustMaterial(null,
                    (x509Certificates, authType) -> true);
            config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(contextBuilder.build())
                    .withReadTimeout(20 * 1000 * 1000).withConnectionTimeout(20 * 1000 * 1000);
        } catch (NoSuchAlgorithmException e) {
            logger.error("NoSuchAlgorithmException", e);
        } catch (KeyStoreException e) {
            logger.error("KeyStoreException", e);
        } catch (KeyManagementException e) {
            logger.error("KeyManagementException", e);
        }
    }

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Value("${collector.openstack.openstack vm.enable:true}")
    boolean enable;

    @Override
    public List<MetricFamilySamples> collect() {
        return super.collect();
    }

    @Scheduled(initialDelay = 3000L, fixedRate = 120000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect openstack vm metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.OPENSTACK);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        // Prometheus Op虚拟机特殊标签
        List<String> labels = Arrays.asList("instanceId","name","vendor_id");
        GaugeMetricFamily status = new GaugeMetricFamily(Constants.PREFIX + "vm_success", "help",
                labels);
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.PREFIX + "collect_time_use", "help",
                Arrays.asList("type"));
        List<MetricFamilySamples> mfs = new ArrayList<>();
        for (CloudVendor cloudVendor : listCloudVendor) {
            try {
                OSClient.OSClientV3 os;
                if ("V3".equals(cloudVendor.getVersion())) {
                    String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                    String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                    String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                    String domainName = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("domainName");
                    String projectName = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("projectName");
                    password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                    Identifier domainIdentifier = Identifier.byName(domainName);
                    os = OSFactory.builderV3().endpoint(address.trim() + "/v3")
                            .credentials(account, password, domainIdentifier)
                            .scopeToProject(Identifier.byName(projectName), domainIdentifier)
                            .withConfig(config)
                            .authenticate();
                } else {
                    logger.error("unsupported version {}", cloudVendor.getVersion());
                    break;
                }
                // 获取虚拟机列表
                List<? extends Server> servers = os.compute().servers().listAll(true);
                logger.info("get {} servers ", servers.size());
                for (Server server : servers) {
                    // 虚拟机链接状态
                    int statusMonitor = server.getStatus()== Server.Status.ACTIVE ? 1 : 0;
                    // 虚拟机uuid
                    String uuid = server.getId();
                    // 虚拟机名称
                    String name = server.getName();
                    status.addMetric(Arrays.asList(uuid,name,String.valueOf(cloudVendor.getId())),statusMonitor);
                }
            } catch (Exception e) {
                logger.error("collect openstack vm metrics error!" + e.getMessage(), e);
            }
        }
        collectTimeUse.addMetric(Collections.singletonList("openstack vm"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(status);
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect openstack vm metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    @Override
    public Map<String, Object> getCache() {
        return this.cache;
    }
}
