package com.bocloud.mcs.collector.openstack;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.common.Constants;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.OSClient;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.common.Identifier;
import org.openstack4j.model.compute.ext.Service;
import org.openstack4j.openstack.OSFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * OpenStack 服务状态
 */
@Component
public class ServiceCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(ServiceCollector.class);

    protected static Config config;

    static {
        // 全部信任 不做身份鉴定
        try {
            SSLContextBuilder contextBuilder = SSLContextBuilder.create().loadTrustMaterial(null,
                    (x509Certificates, authType) -> true);
            config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(contextBuilder.build())
                    .withReadTimeout(20 * 1000 * 1000).withConnectionTimeout(20 * 1000 * 1000);
        } catch (NoSuchAlgorithmException e) {
            logger.error("NoSuchAlgorithmException", e);
        } catch (KeyStoreException e) {
            logger.error("KeyStoreException", e);
        } catch (KeyManagementException e) {
            logger.error("KeyManagementException", e);
        }
    }

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Value("${collector.openstack.service.enable:true}")
    boolean enable;

    @Override
    public List<MetricFamilySamples> collect() {
        return super.collect();
    }

    @Scheduled(initialDelay = 4000L, fixedRate = 60000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        logger.info("collect openstack service metrics start...");
        Date start = new Date();
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.OPENSTACK);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);

        //OpenStack服务状态
        GaugeMetricFamily novaAgentState = new GaugeMetricFamily(Constants.PREFIX + "nova_agent_state", "help",
                Arrays.asList("vendor_id", "vendor_name", "host", "service", "status", "id", "zone"));
        GaugeMetricFamily neutronAgentState = new GaugeMetricFamily(Constants.PREFIX + "neutron_agent_state", "help",
                Arrays.asList("vendor_id", "vendor_name", "host", "service", "adminStateUp", "id"));
        GaugeMetricFamily cinderServiceState = new GaugeMetricFamily(Constants.PREFIX + "cinder_service_state", "help",
                Arrays.asList("vendor_id", "vendor_name", "host", "service", "status", "zone", "id"));
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.PREFIX + "collect_time_use", "help",
                Collections.singletonList("type"));

        List<MetricFamilySamples> mfs = new ArrayList<>();
        mfs.add(novaAgentState);
        mfs.add(neutronAgentState);
        mfs.add(cinderServiceState);
        mfs.add(collectTimeUse);
        for (CloudVendor cloudVendor : listCloudVendor) {
            try {
                OSClient.OSClientV3 os;
                if ("V3".equals(cloudVendor.getVersion())) {
                    String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                    String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                    String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                    String domainName = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("domainName");
                    String projectName = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("projectName");
                    password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                    Identifier domainIdentifier = Identifier.byName(domainName);
                    os = OSFactory.builderV3().endpoint(address.trim() + "/v3")
                            .credentials(account, password, domainIdentifier)
                            .scopeToProject(Identifier.byName(projectName), domainIdentifier)
                            .withConfig(config)
                            .authenticate();
                } else {
                    logger.error("unsupported version {}", cloudVendor.getVersion());
                    break;
                }
                //OpenStack服务状态 nova neutron cinder
                os.compute().services().list().forEach(nova -> novaAgentState.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudVendor.getName(), nova.getHost(), nova.getBinary(), String.valueOf(nova.getStatus()), nova.getId(), nova.getZone()),
                        nova.getState() == Service.State.UP ? 1 : 0));
                os.networking().agent().list().forEach(agent -> neutronAgentState.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudVendor.getName(), agent.getHost(), agent.getBinary(), String.valueOf(agent.getAdminStateUp()), agent.getId()),
                        agent.getAlive() ? 1 : 0));
                os.blockStorage().services().list().forEach(cinder -> cinderServiceState.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudVendor.getName(), cinder.getHost(), cinder.getBinary(), String.valueOf(cinder.getStatus()),
                        cinder.getZone(), cloudVendor.getId() + "_" + cinder.getHost() + "_" + cinder.getBinary()), cinder.getState() == org.openstack4j.model.storage.block.ext.Service.State.UP ? 1 : 0));
            } catch (Exception e) {
                logger.error("collect openstack metrics error!" + e.getMessage(), e);
            }
        }
        collectTimeUse.addMetric(Collections.singletonList("service"), (System.currentTimeMillis() - start.getTime()));
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect openstack service metrics end! use " + (System.currentTimeMillis() - start.getTime()) / 1000 + "s");
    }

    @Override
    public Map<String, Object> getCache() {
        return this.cache;
    }

}
