package com.bocloud.mcs.collector.smis;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.mcs.common.Constants;
import com.bocloud.service.core.RegistryService;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;

/**
 * Dell
 * <p>
 * Copy from SmisServiceHuaWei
 *
 * @see com.bocloud.mcs.collector.smis.SmisServiceHuaWei
 */
@Service("smisService[DELL_CMPL]")
public class SmisServiceDell extends SmisService {

    private final Logger logger = LoggerFactory.getLogger(SmisServiceDell.class);

    private final String smisUriDevice = "/api/pontus/v1/provider/device";


    private final ExecutorService executorService;

    // 针对华为存储返回原始数据进行缓存，华为存储的部分指标需要先后两个原始数据进行计算
    private final Map<String, SmisServiceDell.Calculator> ioTimeCounterTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> totalIOsTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> readIOsTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> writeIOsTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> totalKBTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> readKBTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> writeKBTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> readHitTotalMap = new HashMap<>();
    private final Map<String, SmisServiceDell.Calculator> writeHitTotalMap = new HashMap<>();

    @Autowired
    public SmisServiceDell(RegistryService registryService, @Value("${collector.smis.dell.threadPoolSize:0}") int threadPoolSize) {
        super(registryService);
        ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("smis-dell-pool-%d").build();
        if (threadPoolSize == 0) {
            executorService = new ThreadPoolExecutor(0, Integer.MAX_VALUE,
                    60L, TimeUnit.SECONDS,
                    new SynchronousQueue<>(), nameThreadFactory);
            logger.info("The thread pool size： Integer.MAX_VALUE");
        } else {
            executorService = new ThreadPoolExecutor(threadPoolSize, threadPoolSize,
                    0L, TimeUnit.MILLISECONDS,
                    new LinkedBlockingQueue<>(), nameThreadFactory);
            logger.info("The thread pool size： " + threadPoolSize);
        }
    }

    @Override
    public List<GaugeMetricFamily> getStat(CloudVendor vendor) throws Exception {
        // 累计IO消耗的时间，毫秒，SMIS原始数据：IOTimeCounter
        GaugeMetricFamily ioTimeCounterTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "ioTimeCounter_total", "help", labels);

        // 累计总IO次数，次，SMIS原始数据：TotalIOs
        GaugeMetricFamily totalIOsTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "totalIOs_total", "help", labels);
        // 累计读IO次数，次，SMIS原始数据：ReadIOs
        GaugeMetricFamily readIOsTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readIOs_total", "help", labels);
        // 累计写IO次数，次，SMIS原始数据：WriteIOs
        GaugeMetricFamily writeIOsTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeIOs_total", "help", labels);

        // 累计传输数据，kb，SMIS原始数据：KbytesTransferred
        GaugeMetricFamily totalKBTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "totalKB_total", "help", labels);
        // 累计写数据，kb，SMIS原始数据：KBytesWritten
        GaugeMetricFamily readKBTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readKB_total", "help", labels);
        // 累计读数据，kb，SMIS原始数据：KBytesRead
        GaugeMetricFamily writeKBTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeKB_total", "help", labels);

        // 累计读Cache命中次数，次，SMIS原始数据：ReadHitIOs
        GaugeMetricFamily readHitTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readHit_total", "help", labels);
        // 累计写Cache命中次数，次，SMIS原始数据：WriteHitIOs
        GaugeMetricFamily writeHitTotal = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeHit_total", "help", labels);

        // 平均IO响应时间，毫秒，计算数据：△IOTimeCounter/△TotalIOs
        GaugeMetricFamily totalIOt = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "totalIOt", "help", labels);
        // 平均IO读响应时间，毫秒，计算数据：△IOTimeCounter/△ReadIOs
        GaugeMetricFamily readIOt = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readIOt", "help", labels);
        // 平均IO写响应时间，毫秒，计算数据：△IOTimeCounter/△WriteIOs
        GaugeMetricFamily writeIOt = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeIOt", "help", labels);

        // 总IOPS，IO/秒，计算数据：△TotalIOs/△IOTimeCounter
        GaugeMetricFamily totalIOPS = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "totalIOs", "help", labels);
        // 读IOPS，IO/秒，计算数据：△ReadIOs/△IOTimeCounter
        GaugeMetricFamily readIOPS = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readIOs", "help", labels);
        // 写IOPS，IO/秒，计算数据：△WriteIOs/△IOTimeCounter
        GaugeMetricFamily writeIOPS = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeIOs", "help", labels);

        // 传输数据，kb/秒，计算数据：△KbytesTransferred/时间差
        GaugeMetricFamily totalKBPS = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "totalKB", "help", labels);
        // 写数据，kb/秒，计算数据：△KBytesRead/时间差
        GaugeMetricFamily readKBPS = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readKB", "help", labels);
        // 读数据，kb/秒，计算数据：△KBytesWritten/时间差
        GaugeMetricFamily writeKBPS = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeKB", "help", labels);

        // 读Cache命中率，%，计算数据：△ReadHitIOs/△ReadIOs
        GaugeMetricFamily readHitRate = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "readHit", "help", labels);
        // 写Cache命中率，%，计算数据：△WriteHitIOs/△WriteIOs
        GaugeMetricFamily writeHitRate = new GaugeMetricFamily(Constants.BOCLOUD_SMIS + "writeHit", "help", labels);

        List<GaugeMetricFamily> result = Arrays.asList(
                ioTimeCounterTotal, totalIOsTotal, readIOsTotal, writeIOsTotal, totalKBTotal, readKBTotal, writeKBTotal, readHitTotal, writeHitTotal,
                totalIOt, readIOt, writeIOt, totalIOPS, readIOPS, writeIOPS, totalKBPS, readKBPS, writeKBPS, readHitRate, writeHitRate
        );

        String storageJSON = this.listDevices(vendor);
        if (StringUtils.isEmpty(storageJSON)) {
            logger.debug("Get SMIS[{}] storages empty", vendor.getName());
            return result;
        }
        logger.debug("Get SMIS[{}] storages: {}", vendor.getName(), SmisUtil.trimAny(storageJSON));
        JSONArray storages = JSON.parseArray(storageJSON);
        if (storages.size() == 0) {
            logger.debug("Get SMIS[{}] storages empty", vendor.getName());
            return result;
        }

        // 将所有需要获取监控的任务commit到线程池
        Map<String[], Future<String>> futures = new HashMap<>();
        for (int i = 0; i < storages.size(); i++) {
            JSONObject storage = storages.getJSONObject(i);
            String storageId = DigestUtils.md5Hex(storage.getString("instance_uuid"));
            String storagePath = storage.getString("path");
            String storageName = storage.getString("name");
            // 获取存储设备的监控数据
            SmisCimModel storageCimModel = new SmisCimModel(SmisClass.STORAGE, storagePath, "", SmisClass.STATS);
            SmisStatCallable storageCallable = new SmisStatCallable("SMIS-STORAGE-" + storageName, this, storageCimModel, vendor);
            Future<String> storageFuture = executorService.submit(storageCallable);
            String[] storageCaption = {String.valueOf(vendor.getId()), storageId, storageName, "", "", "storage"};
            futures.put(storageCaption, storageFuture);
            // 获取存储卷的监控数据
            String volumeJSON = super.smisCim(vendor, new SmisCimModel(SmisClass.STORAGE, storagePath, "", SmisClass.VOLUME));
            if (StringUtils.isEmpty(volumeJSON)) {
                logger.debug("Get SMIS[{}]STORAGE[{}] volumes empty", vendor.getName(), storageName);
                continue;
            }
            logger.debug("Get SMIS[{}]STORAGE[{}] volumes: {}", vendor.getName(), storageName, SmisUtil.trimAny(volumeJSON));
            JSONArray volumes = JSON.parseArray(volumeJSON);
            if (volumes.size() == 0) {
                logger.debug("Get SMIS[{}]STORAGE[{}] volumes empty", vendor.getName(), storageName);
                continue;
            }
            for (int j = 0; j < volumes.size(); j++) {
                JSONObject volume = volumes.getJSONObject(j);
                String volumeId = volume.getString("volume_uuid");
                String volumePath = volume.getString("path");
                String volumeName = volume.getString("name");

                SmisCimModel volumeCimModel = new SmisCimModel(SmisClass.VOLUME, volumePath, "", SmisClass.STATS);
                SmisStatCallable volumeCallable = new SmisStatCallable("SMIS-VOLUME-" + volumeName, this, volumeCimModel, vendor);
                Future<String> volumeFuture = executorService.submit(volumeCallable);
                String[] volumeCaption = {String.valueOf(vendor.getId()), volumeId, volumeName, storageId, storageName, "volume"};
                futures.put(volumeCaption, volumeFuture);
            }
            // 获取硬盘的监控数据
            SmisCimModel diskModel = new SmisCimModel();
            diskModel.put("path", storagePath);
            diskModel.put("associator", MapTools.simpleMap("name", SmisClass.DISK));
            String diskJSON = super.smisCim(vendor, diskModel);
            if (StringUtils.isEmpty(diskJSON)) {
                logger.debug("Get SMIS[{}]STORAGE[{}] disks empty", vendor.getName(), storageName);
                continue;
            }
            logger.debug("Get SMIS[{}]STORAGE[{}] disks: {}", vendor.getName(), storageName, SmisUtil.trimAny(diskJSON));
            JSONArray disks = JSON.parseArray(diskJSON);
            if (disks.size() == 0) {
                logger.debug("Get SMIS[{}]STORAGE[{}] disks empty", vendor.getName(), storageName);
                continue;
            }
            for (int j = 0; j < disks.size(); j++) {
                JSONObject disk = disks.getJSONObject(j);
                String diskId = DigestUtils.md5Hex(disk.getString("instance_uuid"));
                String diskPath = disk.getString("path");
                String diskName = disk.getString("name");

                SmisCimModel diskCimModel = new SmisCimModel();
                diskCimModel.put("path", diskPath);
                Map<String, Object> associator = new HashMap<>();
                associator.put("name", SmisClass.DISK_EXTENT);
                Map<String, Object> associatorMap = new HashMap<>();
                associatorMap.put("name", SmisClass.STATS);
                associatorMap.put("assoc_name", SmisClass.ELEMENT_STATS);
                associator.put("associator", associatorMap);
                diskCimModel.put("associator", associator);
                SmisStatCallable diskCallable = new SmisStatCallable("SMIS-DISK-" + diskName, this, diskCimModel, vendor);
                Future<String> diskFuture = executorService.submit(diskCallable);
                String[] diskCaption = {String.valueOf(vendor.getId()), diskId, diskName, storageId, storageName, "disk"};
                futures.put(diskCaption, diskFuture);
            }
        }
        // 从线程池中获取结果
        for (Map.Entry<String[], Future<String>> entry : futures.entrySet()) {
            List<String> captions = Arrays.asList(entry.getKey());
            String instanceUuid = captions.get(labels.indexOf("instanceUuid"));

            String statJSON = entry.getValue().get();
            if (StringUtils.isEmpty(statJSON)) {
                logger.debug("Get SMIS[{}]STAT[{}] empty", vendor.getName(), entry.getKey()[labels.indexOf("instanceName")]);
                continue;
            }
            JSONArray stats = JSON.parseArray(statJSON);
            if (stats.size() == 0) {
                logger.debug("Get SMIS[{}]STAT[{}] empty", vendor.getName(), entry.getKey()[labels.indexOf("instanceName")]);
                continue;
            }
            logger.debug("Get SMIS[{}]STAT[{}]: {}", vendor.getName(), entry.getKey()[labels.indexOf("instanceName")], SmisUtil.trimAny(entry.getValue().get()));

            Date date = new SimpleDateFormat("yyyyMMddHHmmss").parse(stats.getJSONObject(0).getString("stats_time").substring(0, 14));

            double ioTimeCounterTotalValue = stats.getJSONObject(0).getDoubleValue("io_time");
            double totalIOsTotalValue = stats.getJSONObject(0).getDoubleValue("total_ios");
            double readIOsTotalValue = stats.getJSONObject(0).getDoubleValue("read_ios");
            double writeIOsTotalValue = stats.getJSONObject(0).getDoubleValue("write_ios");
            double totalKBTotalValue = stats.getJSONObject(0).getDoubleValue("kb_transferred");
            double readKBTotalValue = stats.getJSONObject(0).getDoubleValue("kb_read");
            double writeKBTotalValue = stats.getJSONObject(0).getDoubleValue("kb_written");
            double readHitTotalValue = stats.getJSONObject(0).getDoubleValue("read_hit_ios");
            double writeHitTotalValue = stats.getJSONObject(0).getDoubleValue("write_hit_ios");

            // 处理原始数据系列指标
            ioTimeCounterTotal.addMetric(captions, ioTimeCounterTotalValue);
            totalIOsTotal.addMetric(captions, totalIOsTotalValue);
            readIOsTotal.addMetric(captions, readIOsTotalValue);
            writeIOsTotal.addMetric(captions, writeIOsTotalValue);
            totalKBTotal.addMetric(captions, totalKBTotalValue);
            readKBTotal.addMetric(captions, readKBTotalValue);
            writeKBTotal.addMetric(captions, writeKBTotalValue);
            readHitTotal.addMetric(captions, readHitTotalValue);
            writeHitTotal.addMetric(captions, writeHitTotalValue);
            // map为空则放到Map
            SmisServiceDell.Calculator ioTimeCounterTotalCalculate = ioTimeCounterTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, ioTimeCounterTotalValue));
            SmisServiceDell.Calculator totalIOsTotalCalculate = totalIOsTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, totalIOsTotalValue));
            SmisServiceDell.Calculator readIOsTotalCalculate = readIOsTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, readIOsTotalValue));
            SmisServiceDell.Calculator writeIOsTotalCalculate = writeIOsTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, writeIOsTotalValue));
            SmisServiceDell.Calculator totalKBTotalCalculate = totalKBTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, totalKBTotalValue));
            SmisServiceDell.Calculator readKBTotalCalculate = readKBTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, readKBTotalValue));
            SmisServiceDell.Calculator writeKBTotalCalculate = writeKBTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, writeKBTotalValue));
            SmisServiceDell.Calculator readHitTotalCalculate = readHitTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, readHitTotalValue));
            SmisServiceDell.Calculator writeHitTotalCalculate = writeHitTotalMap.putIfAbsent(instanceUuid, new SmisServiceDell.Calculator(date, writeHitTotalValue));
            // 计算差值
            Double ioTimeCounterTotalValueDiff = ioTimeCounterTotalCalculate == null ? null : ioTimeCounterTotalValue - ioTimeCounterTotalCalculate.getValue();
            Double totalIOsTotalValueDiff = totalIOsTotalCalculate == null ? null : totalIOsTotalValue - totalIOsTotalCalculate.getValue();
            Double readIOsTotalValueDiff = readIOsTotalCalculate == null ? null : readIOsTotalValue - readIOsTotalCalculate.getValue();
            Double writeIOsTotalValueDiff = writeIOsTotalCalculate == null ? null : writeIOsTotalValue - writeIOsTotalCalculate.getValue();
            Double totalKBTotalValueDiff = totalKBTotalCalculate == null ? null : totalKBTotalValue - totalKBTotalCalculate.getValue();
            Double readKBTotalValueDiff = readKBTotalCalculate == null ? null : readKBTotalValue - readKBTotalCalculate.getValue();
            Double writeKBTotalValueDiff = writeKBTotalCalculate == null ? null : writeKBTotalValue - writeKBTotalCalculate.getValue();
            Double readHitTotalValueDiff = readHitTotalCalculate == null ? null : readHitTotalValue - readHitTotalCalculate.getValue();
            Double writeHitTotalValueDiff = writeHitTotalCalculate == null ? null : writeHitTotalValue - writeHitTotalCalculate.getValue();
            // 计算时间差
            Long ioTimeCounterTotalDateDiff = ioTimeCounterTotalCalculate == null ? null : date.getTime() - ioTimeCounterTotalCalculate.getDate().getTime();
            Long totalIOsTotalDateDiff = totalIOsTotalCalculate == null ? null : date.getTime() - totalIOsTotalCalculate.getDate().getTime();
            Long readIOsTotalDateDiff = readIOsTotalCalculate == null ? null : date.getTime() - readIOsTotalCalculate.getDate().getTime();
            Long writeIOsTotalDateDiff = writeIOsTotalCalculate == null ? null : date.getTime() - writeIOsTotalCalculate.getDate().getTime();
            Long totalKBTotalDateDiff = totalKBTotalCalculate == null ? null : date.getTime() - totalKBTotalCalculate.getDate().getTime();
            Long readKBTotalDateDiff = readKBTotalCalculate == null ? null : date.getTime() - readKBTotalCalculate.getDate().getTime();
            Long writeKBTotalDateDiff = writeKBTotalCalculate == null ? null : date.getTime() - writeKBTotalCalculate.getDate().getTime();
            Long readHitTotalDateDiff = readHitTotalCalculate == null ? null : date.getTime() - readHitTotalCalculate.getDate().getTime();
            Long writeHitTotalDateDiff = writeHitTotalCalculate == null ? null : date.getTime() - writeHitTotalCalculate.getDate().getTime();
            // 处理计算指标
            if (ioTimeCounterTotalValueDiff != null && totalIOsTotalValueDiff != null) {
                totalIOt.addMetric(captions, totalIOsTotalValueDiff == 0 ? 0 : Math.round(ioTimeCounterTotalValueDiff / totalIOsTotalValueDiff * 1000D) / 1000D);
            }
            if (ioTimeCounterTotalValueDiff != null && readIOsTotalValueDiff != null) {
                readIOt.addMetric(captions, readIOsTotalValueDiff == 0 ? 0 : Math.round(ioTimeCounterTotalValueDiff / readIOsTotalValueDiff * 1000D) / 1000D);
            }
            if (ioTimeCounterTotalValueDiff != null && writeIOsTotalValueDiff != null) {
                writeIOt.addMetric(captions, writeIOsTotalValueDiff == 0 ? 0 : Math.round(ioTimeCounterTotalValueDiff / writeIOsTotalValueDiff * 1000D) / 1000D);
            }
            if (totalIOsTotalValueDiff != null && ioTimeCounterTotalValueDiff != null) {
                totalIOPS.addMetric(captions, ioTimeCounterTotalValueDiff == 0 ? 0 : Math.round(totalIOsTotalValueDiff / (ioTimeCounterTotalValueDiff / 1000D) * 1000D) / 1000D);
            }
            if (readIOsTotalValueDiff != null && ioTimeCounterTotalValueDiff != null) {
                readIOPS.addMetric(captions, ioTimeCounterTotalValueDiff == 0 ? 0 : Math.round(readIOsTotalValueDiff / (ioTimeCounterTotalValueDiff / 1000D) * 1000D) / 1000D);
            }
            if (writeIOsTotalValueDiff != null && ioTimeCounterTotalValueDiff != null) {
                writeIOPS.addMetric(captions, ioTimeCounterTotalValueDiff == 0 ? 0 : Math.round(writeIOsTotalValueDiff / (ioTimeCounterTotalValueDiff / 1000D) * 1000D) / 1000D);
            }
            if (totalKBTotalValueDiff != null && totalKBTotalDateDiff != null) {
                totalKBPS.addMetric(captions, totalKBTotalDateDiff == 0 ? 0 : Math.round(totalKBTotalValueDiff / (totalKBTotalDateDiff / 1000D) * 1000D) / 1000D);
            }
            if (readKBTotalValueDiff != null && readKBTotalDateDiff != null) {
                readKBPS.addMetric(captions, readKBTotalDateDiff == 0 ? 0 : Math.round(readKBTotalValueDiff / (readKBTotalDateDiff / 1000D) * 1000D) / 1000D);
            }
            if (writeKBTotalValueDiff != null && writeKBTotalDateDiff != null) {
                writeKBPS.addMetric(captions, writeKBTotalDateDiff == 0 ? 0 : Math.round(writeKBTotalValueDiff / (writeKBTotalDateDiff / 1000D) * 1000D) / 1000D);
            }
            if (readHitTotalValueDiff != null && readIOsTotalValueDiff != null) {
                readHitRate.addMetric(captions, readIOsTotalValueDiff == 0 ? 0 : Math.round(readHitTotalValueDiff / readIOsTotalValueDiff * 1000D) / 1000D);
            }
            if (writeHitTotalValueDiff != null && writeIOsTotalValueDiff != null) {
                writeHitRate.addMetric(captions, writeIOsTotalValueDiff == 0 ? 0 : Math.round(writeHitTotalValueDiff / writeIOsTotalValueDiff * 1000D) / 1000D);
            }
            // 更新到Map
            ioTimeCounterTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, ioTimeCounterTotalValue));
            totalIOsTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, totalIOsTotalValue));
            readIOsTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, readIOsTotalValue));
            writeIOsTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, writeIOsTotalValue));
            totalKBTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, totalKBTotalValue));
            readKBTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, readKBTotalValue));
            writeKBTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, writeKBTotalValue));
            readHitTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, readHitTotalValue));
            writeHitTotalMap.put(instanceUuid, new SmisServiceDell.Calculator(date, writeHitTotalValue));
        }
        return result;
    }

    private String listDevices(CloudVendor vendor) throws Exception {
        String url = getPontusUrl(smisUriDevice);
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("provider", getSmisProviderParam(vendor));
        String requestBody = JSON.toJSONString(requestMap);
        logger.debug("[GET] {} ---> {}", url, requestBody);
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> httpEntity = new HttpEntity<>(requestBody, httpHeaders);
        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);
        String responseBody = SmisUtil.unicodeToString(response.getBody());
        if (response.getStatusCodeValue() != 200) {
            logger.error(String.format("[GET] %s <--- [%s] %s", url, response.getStatusCodeValue(), responseBody));
            return null;
        }
        logger.debug("[GET] {} <--- {}", url, SmisUtil.trimAny(responseBody));
        return responseBody;
    }

    private static class Calculator {
        private final Date date;
        private final double value;

        public Calculator(Date date, double value) {
            this.date = date;
            this.value = value;
        }

        public Date getDate() {
            return date;
        }

        public double getValue() {
            return value;
        }
    }
}
