package com.bocloud.mcs.collector.vmware;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.service.core.CurrentService;
import com.bocloud.vmware.vim25.mo.Datastore;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.ServiceInstance;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.net.URL;
import java.util.*;

/**
 * Vmware DataStore collector
 *
 * @author zhangyf
 * @since 2020-08-03
 */
@Component
public class DataStoreCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(DataStoreCollector.class);

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.vmware.datastore.enable:true}")
    boolean enable;

    @Scheduled(initialDelay = 8000L, fixedRate = 200000L)
    @Async
    @Override
    public void scheduledCollect() {
        if(!enable){
            return;
        }
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect vmware datastore metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.VMWARE);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        logger.info("listCloudVendor.size() + ------>" + listCloudVendor.size());
        logger.info("listCloudVendor + ------>" + listCloudVendor);
        List<MetricFamilySamples> mfs = new ArrayList<>();
        List<String> labels = Arrays.asList("vendor_id", "dsname", "source", "vcenter");
        GaugeMetricFamily capacity = new GaugeMetricFamily("vsphere_datastore_disk_capacity_latest", "help", labels);
        GaugeMetricFamily provisioned = new GaugeMetricFamily("vsphere_datastore_disk_provisioned_latest", "help", labels);
        GaugeMetricFamily used = new GaugeMetricFamily("vsphere_datastore_disk_used_latest", "help", labels);
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily("vsphere_datastore_collect_time_use", "help", Collections.singletonList("type"));
        for (CloudVendor vendor : listCloudVendor) {
            try {
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(vendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(vendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");

                Assert.notNull(address, "VCenter EndPoint is Empty!");
                URL url;
                if (address.contains(":")) {
                    String[] endpoints = address.split(":");
                    Assert.isTrue(Integer.valueOf(endpoints[1]) > 0, "VCenter EndPoint has Wrong Port");
                    url = new URL("https", endpoints[0], Integer.valueOf(endpoints[1]), "/sdk");
                } else {
                    url = new URL("https", address, "/sdk");
                }
                ServiceInstance serviceInstance = new ServiceInstance(url, account, password, true);
                ManagedEntity[] dsEntitys = new InventoryNavigator(serviceInstance.getRootFolder()).searchManagedEntities("Datastore");
                Arrays.stream(Optional.ofNullable(dsEntitys).orElse(new ManagedEntity[]{})).forEach(entity -> {
                            Datastore ds = (Datastore) entity;
                            long capacityValue = ds.getSummary().getCapacity() == 0L ? 0 : ds.getSummary().getCapacity() / 1024;
                            long freeSpaceValue = ds.getSummary().getFreeSpace() == 0L ? 0 : ds.getSummary().getFreeSpace() / 1024;
                            long uncommitted = null == ds.getSummary().getUncommitted() ? 0 : (ds.getSummary().getUncommitted() == 0L ? 0 : ds.getSummary().getUncommitted() / 1024);
                            capacity.addMetric(Arrays.asList(String.valueOf(vendor.getId()), ds.getName(), ds.getName(), address), capacityValue);
                            provisioned.addMetric(Arrays.asList(String.valueOf(vendor.getId()), ds.getName(), ds.getName(), address), (capacityValue - freeSpaceValue) + uncommitted);
                            used.addMetric(Arrays.asList(String.valueOf(vendor.getId()), ds.getName(), ds.getName(), address), capacityValue - freeSpaceValue);
                        }
                );
            } catch (Exception e) {
                logger.error("collect vmware datastore metrics error!" + e.getMessage(), e);
            }
        }
        collectTimeUse.addMetric(Arrays.asList("datastore"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(capacity);
        mfs.add(provisioned);
        mfs.add(used);
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect vmware datastore metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}