package com.bocloud.mcs.collector.vmware;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.service.core.CurrentService;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.*;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.net.URL;
import java.util.*;
import java.util.concurrent.*;

/**
 * Vmware主机监控采集器
 * @author fanzhenxi
 * @version 1.0
 * @date 2021-03-01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class VcenterHostStatusCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(VcenterHostStatusCollector.class);

    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("partition-pool-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.vmware.host.enable:true}")
    boolean enable;

    /**
     * 缓存过期时间
     * @return
     */
    @Override
    public int getExpire() {
        return 60000;
    }

    @Scheduled(initialDelay = 5000L, fixedRate = 60000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect vmware host metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.VMWARE);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        List<String> labels = Arrays.asList("source", "job","vcenter");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily("vsphere_host_collect_time_use", "help", Collections.singletonList("type"));
        List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();
        for (CloudVendor cloudVendor : listCloudVendor) {
            Future<List<MetricFamilySamples>> future = executorService.submit(() -> {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                Assert.notNull(address, "VCenter EndPoint is Empty!");
                // @version 5.6.0-SNAPSHOT添加状态告警
                GaugeMetricFamily status = new GaugeMetricFamily("vsphere_host_success", "help", labels);
                URL url;
                if (address.contains(":")) {
                    String[] endpoints = address.split(":");
                    Assert.isTrue(Integer.parseInt(endpoints[1]) > 0, "VCenter EndPoint has Wrong Port");
                    url = new URL("https", endpoints[0], Integer.parseInt(endpoints[1]), "/sdk");
                } else {
                    url = new URL("https", address, "/sdk");
                }
                ServiceInstance serviceInstance = new ServiceInstance(url, account, password, true);
                ManagedEntity[] hosts = new InventoryNavigator(serviceInstance.getRootFolder()).searchManagedEntities("HostSystem");
                for (ManagedEntity host : hosts) {
                    HostSystem hostSystem = (HostSystem) host;
                    // 获取主机状态
                    // connected  连接正常
                    // notResponding  没有响应
                    // disconnected   断开连接
                    HostSystemConnectionState connectionState = hostSystem.getSummary().getRuntime().getConnectionState();
                    int statusMonitor = connectionState.equals(HostSystemConnectionState.connected) ? 1 : 0;
                    status.addMetric(Arrays.asList(host.getName(),"mcs",address), statusMonitor);
                }
                List<MetricFamilySamples> mfs = new ArrayList<>();
                mfs.add(status);
                return mfs;
            });
            futures.add(future);
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
            try {
                mfs.addAll(stringFuture.get());
            } catch (Exception e) {
                logger.error("collect vmware host metrics error!" + e.getMessage(), e);
            }
        });
        collectTimeUse.addMetric(Collections.singletonList("host"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect vmware host metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
