package com.bocloud.mcs.collector.vmware;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.service.core.CurrentService;
import com.bocloud.vmware.vim25.VirtualMachinePowerState;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.ServiceInstance;
import com.bocloud.vmware.vim25.mo.VirtualMachine;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.net.URL;
import java.util.*;
import java.util.concurrent.*;

/**
 * Vmware Partition collector
 *
 * @author fanzx
 * @since 2021-03-09
 */
@Component
public class VcenterVmStatusCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(VcenterVmStatusCollector.class);

    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("partition-pool-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.vmware.vm.status.enable:true}")
    boolean enable;

    /**
     * 缓存过期时间
     */
    @Override
    public int getExpire() {
        return 60000;
    }

    @Scheduled(initialDelay = 5000L, fixedRate = 60000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect vmware vm status metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.VMWARE);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily("vsphere_vm_partition_collect_time_use", "help", Collections.singletonList("type"));
        List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();
        for (CloudVendor cloudVendor : listCloudVendor) {
            Future<List<MetricFamilySamples>> future = executorService.submit(() -> {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                Assert.notNull(address, "VCenter EndPoint is Empty!");
                // @version 5.6.0-SNAPSHOT添加状态告警
                GaugeMetricFamily status = new GaugeMetricFamily("vsphere_vm_success", "help", Arrays.asList("vmname", "vcenter", "uuid"));
                URL url;
                if (address.contains(":")) {
                    String[] endpoints = address.split(":");
                    Assert.isTrue(Integer.parseInt(endpoints[1]) > 0, "VCenter EndPoint has Wrong Port");
                    url = new URL("https", endpoints[0], Integer.parseInt(endpoints[1]), "/sdk");
                } else {
                    url = new URL("https", address, "/sdk");
                }
                ServiceInstance serviceInstance = new ServiceInstance(url, account, password, true);
                ManagedEntity[] vms = new InventoryNavigator(serviceInstance.getRootFolder()).searchManagedEntities("VirtualMachine");
                for (ManagedEntity vm : vms) {
                    VirtualMachine machine = (VirtualMachine) vm;
                    String uuid = machine.getSummary().getConfig().getUuid();
                    VirtualMachinePowerState powerState = machine.getSummary().getRuntime().getPowerState();
                    int statusMonitor = powerState.equals(VirtualMachinePowerState.poweredOn) ? 1 : 0;
                    status.addMetric(Arrays.asList(machine.getName(), address, uuid), statusMonitor);
                }
                List<MetricFamilySamples> mfs = new ArrayList<>();
                mfs.add(status);
                return mfs;
            });
            futures.add(future);
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
            try {
                mfs.addAll(stringFuture.get());
            } catch (Exception e) {
                logger.error("collect vmware vm status metrics error!" + e.getMessage(), e);
            }
        });
        collectTimeUse.addMetric(Arrays.asList("partition"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect vmware vm status metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
